@@ -1,23 +1,20 @@
 # -*- coding: utf-8 -*-
-
 #
-# gPodder (a media aggregator / podcast client)
+# gPodder - A media aggregator and podcast client
 # Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
+# gPodder is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# gPodder is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
-# MA  02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
 #
@@ -40,6 +37,8 @@ DEFAULT_LENGTH = 60*60*1000
 # command line for mplayer
 MPLAYER_COMMAND = 'mplayer -msglevel all=-1 -identify -vo null -ao null -frames 0 "%s" 2>/dev/null'
 
+from gpodder import util
+
 from liblogger import log
 
 try:
@@ -55,31 +54,15 @@ try:
 except:
     log( '(ipodsync) Could not find pymad.')
 
+# are we going to use eyeD3 for cover art extraction?
+get_coverart = False
+
 try:
     import eyeD3
-    log( '(ipodsync) Found eyeD3')
+    get_coverart = True
+    log( '(ipodsync) Found eyeD3, will try to extract cover art from mp3 metadata')
 except:
-    log( '(ipodsync) Coulld not find eyeD3.')
-
-
-# are we going to use python-id3 for cover art extraction?
-use_pyid3 = False
-
-try:
-    # try to load PyID3
-    import ID3
-    use_pyid3 = True
-    log('(ipodsync) Found PyID3, will try to extract cover art from mp3 metadata')
-except:
-    try:
-        # If you build PyID3 from source it's lowercase (??)
-        import id3
-        ID3 = id3
-        use_pyid3 = True
-        log('(ipodsync) Found PyID3, will try to extract cover art from mp3 metadata')
-    except:
-        log('(ipodsync) PyID3 not found - falling back to channel cover for iPod cover art')
-
+    log( '(ipodsync) Could not find eyeD3 - falling back to channel cover for iPod cover art')
 
 import os
 import os.path
@@ -90,7 +73,6 @@ import time
 import string
 import email.Utils
 
-import liblocaldb
 import libpodcasts
 import libgpodder
 import libconverter
@@ -104,11 +86,11 @@ def ipod_supported():
     return enable_ipod_functions
 
 # file extensions that are handled as video
-video_extensions = [ "mov", "mp4", "m4v" ]
+video_extensions = [ "mov", "mp4", "m4v", "divx" ]
 
 # is mplayer available for finding track length?
 use_mplayer = False
-if not os.system("which mplayer >/dev/null 2>&1"):
+if util.find_command( 'mplayer') != None:
     use_mplayer = True
     log('(ipodsync) Found mplayer, using it to find track length of files')
 else:
@@ -159,17 +141,20 @@ class gPodderSyncMethod:
         if episodes == None:
             episodes = channel
 
-        max = len( episodes)
-        pos = 0
-
-        for episode in episodes:
+        gl = libgpodder.gPodderLib()
+        max = len(episodes)
+        for pos, episode in enumerate(episodes):
             if self.cancelled:
                 return False
             self.set_progress( pos, max)
-            if channel.is_downloaded( episode) and channel.get_file_type( episode) in ( 'audio', 'video' ) and (sync_played_episodes or not channel.is_played( episode)):
-                if not self.add_episode_from_channel( channel, episode):
+            if episode.is_downloaded() and episode.file_type() in ( 'audio', 'video' ) and (sync_played_episodes or not episode.is_played()):
+                added = self.add_episode_from_channel(channel, episode)
+                if added:
+                    if gl.config.on_sync_delete:
+                        log('Removing episode after transfer: %s', episode.title, sender=self)
+                        episode.delete_from_disk()
+                else:
                     return False
-            pos = pos + 1
         self.set_progress( pos, max)
 
         self.set_status( channel = _('Completed %s') % channel.title)
@@ -182,6 +167,11 @@ class gPodderSyncMethod:
 
         if channel.is_music_channel:
             channeltext = _('%s (to "%s")') % ( channel.title, channel.device_playlist_name )
+        
+        gl = libgpodder.gPodderLib()
+        if gl.config.on_sync_mark_played:
+            log('Marking as played on transfer: %s', episode.url, sender = self)
+            gl.history_mark_played(episode.url)
 
         self.set_episode_status( episode.title)
         self.set_channel_status( channeltext)
@@ -200,17 +190,15 @@ class gPodderSyncMethod:
 
 
 class gPodder_iPodSync( gPodderSyncMethod):
-    itdb = None
-    ipod_mount = '' # mountpoint for ipod
-    playlist_name = 'gpodder' # name of playlist to sync to
-    pl_master = None
-    pl_podcasts = None
-
     def __init__( self, callback_progress = None, callback_status = None, callback_done = None):
         if not ipod_supported():
             log( '(ipodsync) iPod functions not supported. (libgpod + eyed3 needed)')
         gl = libgpodder.gPodderLib()
-        self.ipod_mount = gl.ipod_mount
+        self.ipod_mount = gl.config.ipod_mount
+        self.itdb = None
+        self.playlist_name = 'gpodder' # name of playlist to sync to
+        self.pl_master = None
+        self.pl_podcasts = None
         gPodderSyncMethod.__init__( self, callback_progress, callback_status, callback_done)
     
     def open( self):
@@ -311,13 +299,17 @@ class gPodder_iPodSync( gPodderSyncMethod):
                 return True
         
         return False
+    
+    def remove_tracks(self, tracklist=[]):
+        for track in tracklist:
+            log( '(ipodsync) Trying to remove: %s', track.title)
+            self.remove_from_ipod(track, [self.pl_podcasts])
 
     def clean_playlist( self):
         if not ipod_supported():
             return False
-        for track in gpod.sw_get_playlist_tracks( self.pl_podcasts):
-            log( '(ipodsync) Trying to remove: %s', track.title)
-            self.remove_from_ipod( track, [ self.pl_podcasts ])
+
+        return gpod.sw_get_playlist_tracks(self.pl_podcasts)
 
     def set_podcast_flags( self, track, episode):
         if not ipod_supported():
@@ -326,13 +318,13 @@ class gPodder_iPodSync( gPodderSyncMethod):
             # Add blue bullet next to unplayed tracks on 5G iPods
             # (only if the podcast has not been played locally already
             gl = libgpodder.gPodderLib()
-            if gl.history_is_played( episode.url):
+            if gl.history_is_played(episode.url) and not gl.config.on_sync_mark_played:
                 track.mark_unplayed = 0x01
                 # Increment playcount if it's played locally
                 # but still has zero playcount on iPod
                 if track.playcount == 0:
                     track.playcount = 1
-            else:
+            elif track.playcount == 0:
                 track.mark_unplayed = 0x02
 
             # Podcast flags (for new iPods?)
@@ -354,23 +346,25 @@ class gPodder_iPodSync( gPodderSyncMethod):
             return True
         return False
             
-    def set_cover_art(self, track, local_filename):
+    def set_cover_art( self, track, local_filename):
         if not ipod_supported():
             return False
-        if use_pyid3:
-            try:
-                cover_filename = local_filename + '.cover.jpg'
-                id3v2_tags = ID3.ID3v2( local_filename )
-                for frame in id3v2_tags.frames:
-                    if frame.id == 'APIC':
-                        cover_file = file(cover_filename, 'w')
-                        cover_file.write(frame.image)
-                        cover_file.close()
-                if os.path.isfile( cover_filename):
+        if get_coverart:
+            tag = eyeD3.Tag()
+            if tag.link( local_filename):
+                if 'APIC' in tag.frames and len( tag.frames['APIC']) > 0:
+                    # Try to guess the file extension (default to jpg)
+                    extension = 'jpg'
+                    if tag.frames['APIC'][0].mimeType == 'image/png':
+                        extension = 'png'
+                    cover_filename = '%s.cover.%s' ( local_filename, extension )
+                    cover_file = open( cover_filename, 'w')
+                    cover_file.write( tag.frames['APIC'][0].imageData)
+                    cover_file.close()
                     gpod.itdb_track_set_thumbnails( track, cover_filename)
                     return True
-            except:
-                log( '(ipodsync) Error reading ID3v2 information for %s' % ( local_filename, ))
+            else:
+                log( '(ipodsync) Error reading cover art for %s' % ( local_filename, ))
         # If we haven't yet found cover art, fall back to channel cover
         return self.set_channel_art( track, local_filename)
 
@@ -387,10 +381,22 @@ class gPodder_iPodSync( gPodderSyncMethod):
             status_text = _('Already on iPod: %s') % ( episode.title, )
             self.set_status( episode = status_text)
             return True
+
+        original_filename = str( episode.local_filename())
+        local_filename = original_filename
+
+        # Reserve 10 MiB for iTunesDB writing (to be on the safe side)
+        RESERVED_FOR_ITDB = 1024*1024*10
+        space_for_track = util.get_free_disk_space(self.ipod_mount) - RESERVED_FOR_ITDB
+        needed = util.calculate_size(local_filename)
+
+        if needed > space_for_track:
+            log('Not enough space on %s: %s available, but need at least %s', self.ipod_mount, util.format_filesize(space_for_track), util.format_filesize(needed), sender = self)
+            self.errors.append( _('Error copying %s: Not enough free disk space on %s') % (episode.title, self.ipod_mount))
+            self.cancelled = True
+            return False
         
         log( '(ipodsync) Adding item: %s from %s', episode.title, channel.title)
-        original_filename = str(channel.getPodcastFilename( episode.url))
-        local_filename = original_filename
         if libconverter.converters.has_converter( os.path.splitext( original_filename)[1][1:]):
             log('(ipodsync) Converting: %s', original_filename)
             callback_status = lambda percentage: self.set_episode_convert_status( episode.title, percentage)
@@ -475,7 +481,10 @@ class gPodder_iPodSync( gPodderSyncMethod):
         playlist = self.get_playlist_for_channel( channel)
         gpod.itdb_playlist_add_track( playlist, track, -1)
 
-        self.set_cover_art( track, local_filename)
+        try:
+            self.set_cover_art( track, local_filename)
+        except:
+            log( 'Error setting cover art for "%s".', episode.title, sender = self)
 
         # dirty hack to get video working, seems to work
         for ext in video_extensions:
@@ -483,11 +492,11 @@ class gPodder_iPodSync( gPodderSyncMethod):
                 track.filetype = 'm4v' # Doesn't seem to matter if it's mp4 or m4v
                 try:
                     # documented on http://ipodlinux.org/ITunesDB
-                    track.mediatype = 0x00000002
+                    track.mediatype = 0x00000006
                 except:
                     # for old libgpod versions, "mediatype" is "unk208"
                     log( '(ipodsync) Seems like your python-gpod is out-of-date.')
-                    track.unk208 = 0x00000002
+                    track.unk208 = 0x00000006
 
         # if it's a music channel, also sync to master playlist
         if channel.is_music_channel:
@@ -508,6 +517,7 @@ class gPodder_iPodSync( gPodderSyncMethod):
             except:
                 log('(ipodsync) Could not remove temporary file %s', local_filename)
 
+        return True
 
 
 class gPodder_FSSync( gPodderSyncMethod):
@@ -515,37 +525,57 @@ class gPodder_FSSync( gPodderSyncMethod):
 
     def __init__( self, callback_progress = None, callback_status = None, callback_done = None):
         gl = libgpodder.gPodderLib()
-        self.destination = gl.mp3_player_folder
+        self.destination = gl.config.mp3_player_folder
         gPodderSyncMethod.__init__( self, callback_progress, callback_status, callback_done)
         self.can_cancel = True
 
     def open( self):
         gpl = libgpodder.gPodderLib()
-        return gpl.can_write_directory( self.destination)
+        return util.directory_is_writable( self.destination)
     
     def add_episode_from_channel( self, channel, episode):
         allowed_chars = set( string.lowercase + string.uppercase + string.digits + ' _.-')
 
         gPodderSyncMethod.add_episode_from_channel( self, channel, episode)
 
-        folder_src = channel.title
-        folder = ''
-        for ch in folder_src:
-            if ch in allowed_chars:
-                folder = folder + ch
-            else:
-                folder = folder + '_'
-        folder = os.path.join( self.destination, folder)
+        gl = libgpodder.gPodderLib()
 
-        from_file = channel.getPodcastFilename( episode.url)
+        if gl.config.fssync_channel_subfolders:
+            # Add channel title as subfolder
+            folder_src = channel.title
+            folder = ''
+            for ch in folder_src:
+                if ch in allowed_chars:
+                    folder = folder + ch
+                else:
+                    folder = folder + '_'
+            folder = os.path.join( self.destination, folder)
+        else:
+            folder = self.destination
 
-        to_file_src = episode.title + os.path.splitext( from_file)[1].lower()
+        from_file = episode.local_filename()
+
+        filename_base = episode.sync_filename()
+
+        # don't allow extremely long file names: cut the
+        # filename at 50 chars (to make FAT-based drives happy)
+        if len(filename_base) > 50:
+            filename_base = filename_base[:50]
+
+        to_file_src = filename_base + os.path.splitext( from_file)[1].lower()
         to_file = ''
         for ch in to_file_src:
             if ch in allowed_chars:
                 to_file = to_file + ch
             else:
                 to_file = to_file + '_'
+
+        # dirty workaround: on bad (empty) episode titles,
+        # we simply use the from_file basename
+        # (please, podcast authors, FIX YOUR RSS FEEDS!)
+        if os.path.splitext( to_file)[0] == '':
+            to_file = os.path.basename( from_file)
+
         to_file = os.path.join( folder, to_file)
 
         try:
