@@ -1,22 +1,20 @@
-
+# -*- coding: utf-8 -*-
 #
-# gPodder (a media aggregator / podcast client)
+# gPodder - A media aggregator and podcast client
 # Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
+# gPodder is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# gPodder is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
-# MA  02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
 
@@ -111,25 +109,3 @@ class UserAppsReader(object):
         return result
 # end of UserAppsReader
 
-
-def dotdesktop_command( command, filename):
-    # the following flags are specified in the FDO standards for ".desktop" files:
-    # http://standards.freedesktop.org/desktop-entry-spec/desktop-entry-spec-0.9.4.html
-    if command.find( '%U') != -1:
-        # A list of URLs. (we only need one, anyway..)
-        return command.replace( '%U', ( '"file://%s"' % (filename) ) )
-    if command.find( '%u') != -1:
-        # A single URL.
-        return command.replace( '%u', ( '"file://%s"' % (filename) ) )
-    if command.find( '%F') != -1:
-        # A list of files. (we only need one...)
-        return command.replace( '%F', filename )
-    if command.find( '%f') != -1:
-        # A single file name, even if multiple files are selected.
-        return command.replace( '%f', filename )
-
-    # default known-good variant: 1st parameter = filename
-    return '%s "%s"' % ( command, filename )
-# end dotdesktop_command
-
-
